/**
 * An implementation of an undirected graph using an adjacency list 
 * Vertex labels are strings 
 * (c) 2021 Aaron Bauer
 */

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class Graph {
    Map<String, Set<String>> adjList; // maps vertices to a set of their neighbors

    public Graph() {
        adjList = new HashMap<>();
    }
    
    public void addVertex(String v) {
        if (!adjList.containsKey(v)) {
            adjList.put(v, new HashSet<String>());
        }
    }

    public void addEdge(String u, String v) {
        // adding an edge will also add the vertices to the graph if needed
        addVertex(u);
        addVertex(v);
        adjList.get(u).add(v);
        adjList.get(v).add(u);
    }

    // return a set of the vertices in the graph
    public Set<String> V() {
        // construct a new HashSet that's a copy of the key set for adjList
        // so that changes to the returned set don't affect the graph
        return new HashSet<>(adjList.keySet());
    }

    public Iterable<String> neighbors(String v) {
        // construct a new HashSet that's a copy of the set of neighbors
        // so that changes to the returned set don't affect the graph
        return new HashSet<>(adjList.get(v));
    }

    public void removeVertex(String v) {
        for (String u : neighbors(v)) {
            adjList.get(u).remove(v);
        }
        adjList.remove(v);
    }
    
    public void removeEdge(String u, String v) {
        adjList.get(u).remove(v);
        adjList.get(v).remove(u);
    }

    public boolean containsVertex(String v) {
        return adjList.containsKey(v);
    }

    public boolean containsEdge(String u, String v) {
        return containsVertex(u) && containsVertex(v) && adjList.get(u).contains(v);
    }
}  